--# Copyright (C) @BABOLO  2002..2008 http://www.babolo.ru/
--# Copyright (C) GD 2005..2007 <gd@pikenet.ru>
--# All rights reserved.
--#
--# Redistribution and use in source and binary forms, with or without
--# modification, are permitted provided that the following conditions
--# are met:
--# 1. Redistributions of source code must retain the above copyright
--#    notice, this list of conditions and the following disclaimer.
--# 2. Redistributions in binary form must reproduce the above copyright
--#    notice, this list of conditions and the following disclaimer in the
--#    documentation and/or other materials provided with the distribution.
--#
--# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
--# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
--# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
--# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
--# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
--# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
--# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
--# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
--# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
--# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
--# SUCH DAMAGE.
--#
--# $Id: pglib_text.sql.m4,v 1.1 2008/01/23 23:47:54 babolo Exp $

CREATE OR REPLACE FUNCTION quote_html(text) RETURNS text
 AS '$libdir/pglib_text.so.VMAJOR', 'quote_html'
 LANGUAGE C IMMUTABLE STRICT
;
CREATE OR REPLACE FUNCTION quote_sh(text) RETURNS text
 AS '$libdir/pglib_text.so.VMAJOR', 'quote_sh'
 LANGUAGE C IMMUTABLE STRICT
;

--#
CREATE OR REPLACE FUNCTION printf(text, text[]) RETURNS text
 AS 'DECLARE loopn integer := 1;
	     result text := '''';
     BEGIN
	result := REPLACE($1, ''%%'', ''%'' || chr(168));
        WHILE $2[loopn] IS NOT NULL LOOP
              result := REPLACE(result, ''%'' || loopn, $2[loopn]);
              loopn := loopn + 1;
        END LOOP;
        result := REPLACE(result, ''%'' || chr(168), ''%'');
    	RETURN result;
     END;
    '
 LANGUAGE PLPGSQL IMMUTABLE
;
COMMENT ON FUNCTION printf(text, text[]) IS '  $1 %  $2[]. %  %.';

--#  
CREATE AGGREGATE cat
     ( BASETYPE = text
     , SFUNC = textcat
     , STYPE = text
     , INITCOND = ''
);
COMMENT ON AGGREGATE cat(text)           IS ' ';

--#
CREATE OR REPLACE FUNCTION array0(anyelement) RETURNS anyarray
 AS 'DECLARE a $0%TYPE:=''{}'';
     BEGIN
        a[0] := $1;
        RETURN a;
     END;
    '
 LANGUAGE PLPGSQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION array0(anyelement)               IS '    [0] ';

CREATE OR REPLACE FUNCTION compare(integer, text[], text[]) RETURNS bool
 AS 'SELECT CASE WHEN $1 IS NULL
                 THEN FALSE
                 WHEN $2 IS NULL AND $3 IS NULL
                 THEN TRUE
                 WHEN $2 IS NULL OR $3 IS NULL
                 THEN FALSE
                 WHEN array_lower($2, 1) <> array_lower($3, 1)
                   OR (   array_upper($2, 1) <> array_upper($3, 1)
                      AND (array_upper($2, 1) < $1 OR array_upper($3, 1) < $1)
                      )
                 THEN FALSE
                 WHEN array_lower($2, 1) > $1
                 THEN TRUE
                 WHEN array_upper($2, 1) > $1
                 THEN $2[array_lower($2, 1):$1] = $3[array_lower($3, 1):$1]
                 ELSE $2 = $3
            END
    '
 LANGUAGE SQL IMMUTABLE
;
COMMENT ON FUNCTION compare(integer, text[], text[]) IS '  $1   $2  $3';

CREATE OR REPLACE FUNCTION cmpstrarrlt(text[], text[]) RETURNS bool
 AS 'DECLARE loopn integer := 1;
             result bool := FALSE;
     BEGIN
         LOOP
             IF $1[loopn] IS NULL AND $2[loopn] IS NULL THEN EXIT; END IF;
             IF $1[loopn] <> $2[loopn]
             THEN result := ($1[loopn] < $2[loopn]); EXIT;
             END IF;
             loopn := loopn + 1;
         END LOOP;
         RETURN result;
     END;
    '
 LANGUAGE PLPGSQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION cmpstrarrlt(text[], text[])      IS '  <   ';

CREATE OR REPLACE FUNCTION cmpstrarrgt(text[], text[]) RETURNS bool
 AS 'SELECT cmpstrarrlt($2, $1)'
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION cmpstrarrgt(text[], text[])      IS '  >   ';

CREATE OR REPLACE FUNCTION cmpstrarrge(text[], text[]) RETURNS bool
 AS 'SELECT NOT cmpstrarrlt($1, $2)'
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION cmpstrarrge(text[], text[])      IS '  >=   ';

CREATE OR REPLACE FUNCTION cmpstrarrle(text[], text[]) RETURNS bool
 AS 'SELECT NOT cmpstrarrlt($2, $1)'
 LANGUAGE SQL IMMUTABLE STRICT
;
COMMENT ON FUNCTION cmpstrarrle(text[], text[])      IS '  <=   ';

CREATE OPERATOR <
     ( LEFTARG = text[]
     , RIGHTARG = text[]
     , PROCEDURE = cmpstrarrlt
     , COMMUTATOR = >
     , NEGATOR = >=
);

CREATE OPERATOR >
     ( LEFTARG = text[]
     , RIGHTARG = text[]
     , PROCEDURE = cmpstrarrgt
     , COMMUTATOR = <
     , NEGATOR = <=
);

CREATE OPERATOR >=
     ( LEFTARG = text[]
     , RIGHTARG = text[]
     , PROCEDURE = cmpstrarrge
     , COMMUTATOR = <=
     , NEGATOR = <
);

CREATE OPERATOR <=
     ( LEFTARG = text[]
     , RIGHTARG = text[]
     , PROCEDURE = cmpstrarrle
     , COMMUTATOR = >=
     , NEGATOR = >
);
